<?php
/**
 * This is the entrypoint for the locrMAPS WordPress plugin.
 *
 * @package locrMAPS
 */

declare(strict_types=1);

namespace Locr\WordPress\Plugin\LocrMaps\DDD\ValueObjects\OnlineMap\Location;

use Locr\WordPress\Plugin\LocrMaps\DDD\ValueObjects\OnlineMap\Location\Address\{City, Country, House, State, Street, Postal};

if ( ! defined( 'ABSPATH' ) ) {
	exit; // Exit if accessed directly.
}

/**
 * This class is responsible for the location address.
 *
 * @property-read City $city The city.
 * @property-read Country $country The country.
 * @property-read House $house The house.
 * @property-read State $state The state.
 * @property-read Street $street The street.
 * @property-read Postal $postal The postal.
 */
final class Address {
	/**
	 * The constructor.
	 *
	 * @param Country $country The country of the address.
	 * @param State   $state The state of the address.
	 * @param City    $city The city of the address.
	 * @param Postal  $postal The postal of the address.
	 * @param Street  $street The street of the address.
	 * @param House   $house The house of the address.
	 */
	public function __construct(
		private Country $country,
		private State $state,
		private City $city,
		private Postal $postal,
		private Street $street,
		private House $house,
	) {
	}

	/**
	 * The magick getter.
	 *
	 * @param string $name The name of the property.
	 * @throws \InvalidArgumentException If the property does not exist.
	 */
	public function __get( string $name ): mixed {
		return match ( $name ) {
			'city' => $this->city,
			'country' => $this->country,
			'house' => $this->house,
			'state' => $this->state,
			'street' => $this->street,
			'postal' => $this->postal,
			default => throw new \InvalidArgumentException( 'The property "' . esc_html( $name ) . '" does not exist.' ),
		};
	}

	/**
	 * Builds an address instance from a JSON string.
	 *
	 * @param mixed $json The JSON string.
	 * @throws \InvalidArgumentException If the JSON is not a string or could not be decoded.
	 */
	public static function from_json( mixed $json ): self {
		if ( ! is_string( $json ) ) {
			throw new \InvalidArgumentException( 'The JSON must be a string.' );
		}

		$data = json_decode( $json, true );
		if ( ! is_array( $data ) ) {
			throw new \InvalidArgumentException( 'The string could not been decoded as JSON.' );
		}

		return new self(
			country: isset( $data['country'] ) ? new Country( $data['country'] ) : new Country(),
			state: isset( $data['state'] ) ? new State( $data['state'] ) : new State(),
			city: isset( $data['city'] ) ? new City( $data['city'] ) : new City(),
			postal: isset( $data['postal'] ) ? new Postal( $data['postal'] ) : new Postal(),
			street: isset( $data['street'] ) ? new Street( $data['street'] ) : new Street(),
			house: isset( $data['house'] ) ? new House( $data['house'] ) : new House(),
		);
	}

	/**
	 * Converts the address to an array.
	 *
	 * @return array<mixed> The address as an array.
	 */
	public function to_array(): array {
		return array(
			'country' => $this->country->value,
			'state'   => $this->state->value,
			'city'    => $this->city->value,
			'postal'  => $this->postal->value,
			'street'  => $this->street->value,
			'house'   => $this->house->value,
		);
	}

	/**
	 * Converts the address to a JSON string.
	 *
	 * @throws \RuntimeException If the address could not be converted to JSON.
	 */
	public function to_json(): string {
		$json = wp_json_encode( $this->to_array() );
		if ( false === $json ) {
			throw new \RuntimeException( 'The address could not be converted to JSON.' );
		}

		return $json;
	}
}
