<?php
/**
 * This is the entrypoint for the locrMAPS WordPress plugin.
 *
 * @package locrMAPS
 */

declare(strict_types=1);

namespace Locr\WordPress\Plugin\LocrMaps\Frontend;

use Locr\WordPress\Plugin\LocrMaps\DDD\Entities\OnlineMap\Location;
use Locr\WordPress\Plugin\LocrMaps\DDD\Repositories\{OnlineMapLocations as OnlineMapLocationsRepository, OnlineMaps as OnlineMapsRepository};
use Locr\WordPress\Plugin\LocrMaps\DDD\ValueObjects\GeneralOptions;
use Locr\WordPress\Plugin\LocrMaps\DDD\ValueObjects\OnlineMap\{BorderType, Height, Id, Width};
use Locr\WordPress\Plugin\LocrMaps\Plugin;

if ( ! defined( 'ABSPATH' ) ) {
	exit; // Exit if accessed directly.
}

/**
 * This class is responsible for the shortcode.
 */
final class Shortcode {
	/**
	 * This function initializes the shortcode.
	 */
	public static function init(): void {
		add_shortcode( Plugin::SLUG, array( self::class, 'render_shortcode' ) );
		add_shortcode( Plugin::SLUG . '_search', array( self::class, 'render_search_shortcode' ) );
	}

	/**
	 * This function renders the shortcode.
	 *
	 * @param array<mixed> $atts The attributes.
	 */
	public static function render_shortcode( array $atts = array() ): string {
		wp_enqueue_script( handle: 'wp-api-fetch' );

		$locrmap_custom_element_version = '5241120';
		wp_enqueue_script_module(
			id: 'locr-map-custom-element',
			src: "https://cdn.jsdelivr.net/gh/locr-company/Leaflet.GridLayer.VMS2@{$locrmap_custom_element_version}/src/LocrMapCustomElement.js",
			version: $locrmap_custom_element_version,
		);
		wp_enqueue_script_module(
			id: Plugin::SLUG . '-custom-element-init',
			src: plugin_dir_url( dirname( __DIR__ ) ) . 'public/js/custom_element_init.js',
			version: md5_file( plugin_dir_path( dirname( __DIR__ ) ) . 'public/js/custom_element_init.js' )
		);

		$onlinemap = null;
		$locations = null;
		if ( isset( $atts['id'] ) && is_string( $atts['id'] ) ) {
			$onlinemaps_repository = new OnlineMapsRepository();
			$onlinemap             = $onlinemaps_repository->get_by_id( new Id( $atts['id'] ) );
		}
		if ( ! is_null( $onlinemap ) ) {
			$locations_repository = new OnlineMapLocationsRepository();
			$locations            = $locations_repository->get_by_id( $onlinemap->id );
		}

		$general_options = new GeneralOptions( $atts );

		$width       = Width::DEFAULT;
		$height      = Height::DEFAULT;
		$max_width   = '100%';
		$max_height  = '100%';
		$border_type = BorderType::None;
		if ( ! is_null( $onlinemap ) ) {
			if ( ! empty( (string) $onlinemap->width ) ) {
				$width = (string) $onlinemap->width;
			}
			if ( ! empty( (string) $onlinemap->height ) ) {
				$height = (string) $onlinemap->height;
			}
			$border_type = $onlinemap->border_type;
		}
		if ( isset( $atts['width'] ) && is_string( $atts['width'] ) ) {
			$width = $atts['width'];
		}
		if ( isset( $atts['height'] ) && is_string( $atts['height'] ) ) {
			$height = $atts['height'];
		}
		if ( isset( $atts['max-width'] ) && is_string( $atts['max-width'] ) ) {
			$max_width = $atts['max-width'];
		}
		if ( isset( $atts['max-height'] ) && is_string( $atts['max-height'] ) ) {
			$max_height = $atts['max-height'];
		}
		if ( isset( $atts['border-type'] ) && is_string( $atts['border-type'] ) ) {
			$border_type = BorderType::tryFrom( $atts['border-type'] ) ?? BorderType::None;
		}

		if ( is_numeric( $width ) ) {
			$width .= 'px';
		}
		if ( is_numeric( $height ) ) {
			$height .= 'px';
		}

		$border_style = '';
		switch ( $border_type ) {
			case BorderType::None:
				$border_style = ' border: none;';
				break;
			case BorderType::Solid:
				$border_style = ' border: 1px solid black;';
				break;
			case BorderType::Shadow:
				$border_style = ' box-shadow: 5px 5px 5px grey;';
				break;
		}

		$rendered = '<div id="' . Plugin::SLUG . '-container" style="width: ' . esc_attr( $width ) . '; height: ' . esc_attr( $height ) . '; max-width: ' . esc_attr( $max_width ) . '; max-width: ' . esc_attr( $max_height ) . '; display: flex;">' . PHP_EOL;

		$rendered .= '<locr-map id="' . Plugin::SLUG . '-map" style="width: ' . esc_attr( $width ) . '; height: ' . esc_attr( $height ) . '; min-width: 240px; min-height: 85px; max-width: 100%; max-height: 100%; margin: auto;' . $border_style . '"' . PHP_EOL;
		if ( ! empty( (string) $general_options->access_key ) ) {
			$rendered .= ' access-key="' . $general_options->access_key . '"';
		}
		if ( ! is_null( $onlinemap ) ) {
			$rendered .= ' map-style="' . (string) $onlinemap->style . '"';
		}
		if ( ! is_null( $locations ) ) {
			$locations_array = array();
			foreach ( $locations as $location ) {
				if ( $location instanceof Location ) {
					$locations_array[] = $location->to_array();
				}
			}
			$json_locations = wp_json_encode( $locations_array );
			if ( false === $json_locations ) {
				$json_locations = '[]';
			}
			$rendered .= ' data-locations="' . esc_attr( $json_locations ) . '"';
		}
		$rendered .= '>';
		$rendered .= '</locr-map>' . PHP_EOL;

		$rendered .= '</div>' . PHP_EOL;

		return $rendered;
	}

	/**
	 * This function renders the search shortcode.
	 *
	 * @param array<mixed> $atts The attributes.
	 */
	public static function render_search_shortcode( array $atts = array() ): string {
		wp_enqueue_script_module(
			id: Plugin::SLUG . '-search-init',
			src: plugin_dir_url( dirname( __DIR__ ) ) . 'public/js/search_init.js',
			version: md5_file( plugin_dir_path( dirname( __DIR__ ) ) . 'public/js/search_init.js' )
		);

		$rendered  = '<input type="text" placeholder="' . esc_attr__( 'Search', 'default' ) . '" id="' . Plugin::SLUG . '-search" class="input-text"';
		$rendered .= '/>';

		return $rendered;
	}
}
